#!/usr/bin/python
"""
Author: Anja Spang
will extract specific sequences listed in a text file from a multifasta-file
this script allows to either extract all sequences with a specific match in the header (requires -e flag set to "all") or to use exact headers (looks only for first occurrence of hit in this second case) (requires -e to be set to "exact")
"""


import sys
import os
import argparse
from Bio import SeqIO

parser = argparse.ArgumentParser(description='extract specific sequences listed in a text file from a multifasta-file')
parser.add_argument('-l','--input1', metavar='TXT', required = True, help='text file with (parts of) sequence headers line by line')
parser.add_argument('-m','--input2', metavar='FASTA', required = True, help='multifasta file from which sequences will be extracted')
parser.add_argument('-e','--input3', metavar='str', required = True, help='takes 2 arguments: "all" or "exact"')
parser.add_argument('-o','--output', metavar='FASTA', help='multifasta file with selected sequences', required = True)
args = parser.parse_args()

if args.input1 is None and not os.path.exists(args.input):
    print("input1 file missing!")
    sys.exit(-1)

if args.input2 is None and not os.path.exists(args.input):
    print("input2 file missing!")
    sys.exit(-1)

if args.input3 is None and not os.path.exists(args.input):
    print("specify -e: 'all' or 'exact' match!")
    sys.exit(-1)

acc_list = [entry.strip() for entry in open(args.input1).readlines()]
Infile = args.input2
mode = str(args.input3)
Outfile = open(args.output, "w")



#Function to extract sequences with specific headers
##########################################################################################

def extract_specific_seq(multifasta, accession_list, MODE):
	for seq_record in SeqIO.parse("%s" % (multifasta),"fasta"):
		header = str(seq_record.description).strip()
		seq = str(seq_record.seq).strip()
		if MODE == 'all':
			for acc in accession_list:
				if acc in header:
					Outfile.write(">%s\n%s\n" % (header, seq))
		if MODE == 'exact':
			for acc in accession_list:
				if acc in header:
					Outfile.write(">%s\n%s\n" % (header, seq))
					acc_list.remove(acc)
					break



#Call main
##########################################################################################

def main():
	extract_specific_seq(Infile, acc_list, mode)

if __name__ == "__main__":
	main()




#Close files
##########################################################################################
Outfile.close()
