#!/usr/bin/python
#Anja, February 2017
#for python3.5
##########################################################################################

#Parse annotation, blast results, orthologous groups, arCOGs, IPR- and PFAM domains and KO numbers
#infiles look as following: (tab separated)
#Annotation; accession	org	locus_tag	product	length
#Blast results: accession	blast-hit	length	e-value score	ID	(blast-hit	length	e-value	score	ID)
#arCOG assignments: accession	arCOGs	COGs	cat	gene	Arcog-annotation
#OrthoMCL results: accession	orthologousGroup
#IPR, PFAM, KO results: accession	IPRdomain	IPRdes	PFAMdomain	PFAMdes	ECnumber




#Import modules
##########################################################################################

import sys
import os
import re
import argparse
import pandas as pd




#ARGPARSE
##########################################################################################

parser = argparse.ArgumentParser(description='Create annotation file with one gene per line listing prokka annotations with corresponding Blast results, arCOGs, IPR domains, pfam domains and EC-numbers')
parser.add_argument('-an','--input1', metavar='tsv', required = True, help='Parsed annotation, tsv file')
parser.add_argument('-bl','--input2', metavar='tsv', required = True, help='Parsed blast hits: generated with Parse_diamond_blast_results.py')
parser.add_argument('-ar','--input3', metavar='tsv', required = True, help='Parsed arCOG asignments: generated with Parse_arCOG_assignments.py')
parser.add_argument('-or','--input4', metavar='tsv', required = True, help='Parsed OrthoMCL assignments: generated with Parse_orthologousGroups.py')
parser.add_argument('-ip','--input5', metavar='tsv', required = True, help='Parsed IPR, PFAM and KO asignments: generated with Parse_IPRdomains_vs2.py')
parser.add_argument('-o','--output', metavar='tsv', help= 'protein annotation file', required = True)
args = parser.parse_args()



if args.input1 is None and not os.path.exists(args.input):
  print("gene product file missing!")
  sys.exit(-1)

if args.input2 is None and not os.path.exists(args.input):
    print("blast results file missing!")
    sys.exit(-1)

if args.input3 is None and not os.path.exists(args.input):
	    print("arCOG results file missing!")
	    sys.exit(-1)

if args.input4 is None and not os.path.exists(args.input):
    print("OrthoMCL results file missing!")
    sys.exit(-1)

if args.input5 is None and not os.path.exists(args.input):
    print("IPRscan results file missing!")
    sys.exit(-1)




#Open and read files
##########################################################################################
infile1 = open(args.input1).readlines()
infile2 = open(args.input2).readlines()
infile3 = open(args.input3).readlines()
infile4 = open(args.input4).readlines()
infile5 = open(args.input5).readlines()

outfile = open("%s" % args.output,'w')



#Save files as dictionaries
##########################################################################################

def Parse_Accession_Protein_dict(infile1):
	Protein_dict = {}
	accessions_list = []

	for line in infile1:
		entry = line.split('\t', 1)
		accessions_list.append(entry[0].strip())
		Protein_dict[entry[0]] = entry[1].strip()

	return Protein_dict, accessions_list



def parseBlast_dict(infile2):
	Blast_dict = {}

	for line in infile2:
		entry = line.split('\t', 1)
		Blast_dict[entry[0]] = entry[1].strip()

	return Blast_dict



def parseArcog_dict(infile3):
	ArCOG_dict = {}

	for line in infile3:
		entry = line.split('\t', 1)
		ArCOG_dict[entry[0]] = entry[1].strip()

	return ArCOG_dict


def parseOMCL_dict(infile4):
	OMCL_dict = {}

	for line in infile4:
		entry = line.split('\t', 1)
		OMCL_dict[entry[0]] = entry[1].strip()

	return OMCL_dict


def parseIPR_dict(infile5):
	IPR_dict = {}

	for line in infile5:
		entry = line.split('\t', 1)
		IPR_dict[entry[0]] = entry[1].strip()

	return IPR_dict




#Parse final annotation table
##########################################################################################
Protein_dict, Accessions_list = Parse_Accession_Protein_dict(infile1)
Blast_dict = parseBlast_dict(infile2)
ArCOG_dict = parseArcog_dict(infile3)
OMCL_dict = parseOMCL_dict(infile4)
IPR_dict = parseIPR_dict(infile5)


def Parse_final(Protein_dict, Blast_dict, ArCOG_dict, OMCL_dict, IPR_dict):
	outfile.write("accession\torganism\tLocus-tag\tproduct\tprot-length\tTopHit\tE-value\tscore\tid\tTopHit-wo\tE-value-wo\tscore-wo\tID-wo\tarCOG\tCOG\tcat\tgene\tarcog-annotation\tOrthol.Group\tIPRdomain\tIPRdesc\tPFAMdomain\tPFAMdesc\tPathway\n")

	for acc in Accessions_list:

		if acc in Protein_dict:
			product = Protein_dict[acc]
		else:
			product = "-"

		if acc in Blast_dict:
			Blast_results = Blast_dict[acc]
		else:
			Blast_results = "-\t-\t-\t-\t-\t-\t-\t-"

		if acc in ArCOG_dict:
			arcog_results = ArCOG_dict[acc]
		else:
			arcog_results = "-\t-\t-\t-\t-"

		if acc in OMCL_dict:
			omcl_results = OMCL_dict[acc]
		else:
			omcl_results = "-"

		if acc in IPR_dict:
			ipr_results = IPR_dict[acc]
		else:
			ipr_results = "-\t-\t-\t-\t-"


		outfile.write("%s\t%s\t%s\t%s\t%s\t%s\n" %(acc, product, Blast_results, arcog_results, omcl_results, ipr_results))



def main():
	Parse_final(Protein_dict, Blast_dict, ArCOG_dict, OMCL_dict, IPR_dict)


if __name__ == "__main__":
	main()



outfile.close()
