#!/usr/bin/python
"""
Author: Anja Spang
will check lenght of fasta seq and remove sequences shorter than desired lenght
will  print second file
"""

#LOAD MODULES
#####################################################################################################
import sys
import os
import argparse
from Bio import SeqIO



#ARGPARSE and variable assignment
#####################################################################################################
parser = argparse.ArgumentParser(description='extract sequences with a pecific length cutoff, both max and min AA length can be given')
parser.add_argument('-i','--input1', metavar='FASTA', required = True, help='multifasta file from which sequences will be extracted')
parser.add_argument('-min','--input2', metavar= str, required = False, help='length min cutoff')
parser.add_argument('-max','--input3', metavar= str, required = False, help='length max cutoff')
parser.add_argument('-o1','--output1', metavar='FASTA', help='multifasta file with selected sequences', required = True)
parser.add_argument('-o2','--output2', metavar='FASTA', help='multifasta file with removed sequences', required = True)
parser.add_argument('-l','--output3', metavar='tab', help='tab-deliminted file with all accessions and length information ', required = True)
args = parser.parse_args()

if args.input1 is None and not os.path.exists(args.input):
    print("fasta file missing!")
    sys.exit(-1)

if args.input2 is None and args.input3 is None:
    print("specify either min, max or both")
    sys.exit(-1)
   

if args.input2 is None:
        length_cutoff_min = 0
else:
        length_cutoff_min =  int(args.input2)


if args.input3 is None:
        length_cutoff_max = 0
else:
        length_cutoff_max =  int(args.input3)



#Read and open files:
#####################################################################################################

Infile = args.input1
Outfile1 = open(args.output1, "w")
Outfile2 = open(args.output2, "w")
Outfile3 = open(args.output3, "w")



#Function to extract seq based on a user provided min length cutoff
#####################################################################################################

def remove_seq(multifasta, lencutoffmin, lencutoffmax):
	for seq_record in SeqIO.parse(multifasta,"fasta"):
		 
		#only minimum length cutoff given
		if lencutoffmax == 0 and lencutoffmin != 0:
			header = str(seq_record.description).strip()
			seq = str(seq_record.seq).strip()
			length = int(len(seq))
			if length > lencutoffmin:
				Outfile1.write(">%s\n%s\n" % (header, seq))
			else:	
				Outfile2.write(">%s\n%s\n" % (header, seq))
			Outfile3.write("%s\t%s\n" % (header, length))

		#only maximum length cutoff given
		elif lencutoffmax != 0 and lencutoffmin == 0:		
			header = str(seq_record.description).strip()
			seq = str(seq_record.seq).strip()
			length = int(len(seq))
			if length < lencutoffmax:
				Outfile1.write(">%s\n%s\n" % (header, seq))
			else:
				Outfile2.write(">%s\n%s\n" % (header, seq)) 
			Outfile3.write("%s\t%s\n" % (header, length))
				
		#both a minimum and maximum lengthcutoff given:
		elif lencutoffmax != 0 and lencutoffmin != 0:
			header = str(seq_record.description).strip()
			seq = str(seq_record.seq).strip()
			length = int(len(seq))
			if length > lencutoffmin and length < lencutoffmax:
				Outfile1.write(">%s\n%s\n" % (header, seq))
			else:
				Outfile2.write(">%s\n%s\n" % (header, seq))
			Outfile3.write("%s\t%s\n" % (header, length))
				



#Call main
##########################################################################################

def main():
	remove_seq(Infile, length_cutoff_min, length_cutoff_max)

if __name__ == "__main__":
	main()



#Close files
##########################################################################################

Outfile1.close()
Outfile2.close()


