#!/usr/bin/python
"""
Author: Anja Spang
Change headers so that accession numbers and species names are kept
"""

#####################################################################################################3

import sys
import os
import argparse
from Bio import SeqIO
import re

#####################################################################################################3

parser = argparse.ArgumentParser(description='modifies headers in a multifasta file')
parser.add_argument('-a','--input1', metavar='FASTA', required = True, help='multifasta file')
parser.add_argument('-o','--output', metavar='FASTA', help='multifasta file with modified headers of the format: accession|species (headers from prodigal that inlcude # are only trimmed', required = True)
parser.add_argument('-del','--delimiter', required = True, help='delimiter to separate accession from species')
args = parser.parse_args()

if args.input1 is None and not os.path.exists(args.input):
    print("multifasta file missing!")
    sys.exit(-1)


#####################################################################################################3

infile = args.input1
outfile = open(args.output, "w")
delimiter_char= str(args.delimiter)

#####################################################################################################3

for seq_record in SeqIO.parse("%s" % (infile),"fasta"):
	header = str(seq_record.description).strip()
	seq = str(seq_record.seq).strip()

	if ">" in header:
		header = header.split('>')[0]

	try:
		header_accession = header.split(' ')[0]
		header_taxa = header.split('[')[1].split(']')[0].replace(' ','_').replace('.','')
		outfile.write(">%s%s%s\n%s\n" % (header_accession, delimiter_char, header_taxa, seq))
	except:
		outfile.write(">%s\n%s\n" % (header, seq))



outfile.close()
