#!/usr/bin/python
"""
Author: Anja Spang
Change headers so that accession numbers and species names are kept while gi numbers and descriptions are removed
"""


import sys
import os
import argparse
from Bio import SeqIO
import re


parser = argparse.ArgumentParser(description='modifies uniprot headers in a multifasta file')
parser.add_argument('-i','--input1', metavar='FASTA', required = True, help='multifasta file')
parser.add_argument('-o','--output', metavar='FASTA', help='multifasta file with modified headers of the format: accession|species', required = True)
parser.add_argument('-d','--delimiter', required = True, help='delimiter to separate accession from species')
args = parser.parse_args()


if args.input1 is None and not os.path.exists(args.input):
    print("input1 file missing!")
    sys.exit(-1)



infile = args.input1
outfile = open(args.output, "w")
delimiter_char= str(args.delimiter)


for seq_record in SeqIO.parse("%s" % (infile),"fasta"):
    header_old = str(seq_record.description).strip()
    seq = str(seq_record.seq).strip()
    header_accession = header_old.split("|")[1]
    if "GN=" in header_old:
      header_org = ((((((((((((header_old.split("OS=")[1]).split(" GN=")[0]).replace("/","-")).replace("(","")).replace(")","")).replace(":","_")).replace(".","")).replace("=","_")).replace(" ", "_")).replace("__","_")).replace("_-_","_")).replace("'","")).replace("#","")
      outfile.write(">%s%s%s\n%s\n" % (header_accession, delimiter_char, header_org, seq))
    else:
      header_org = (((((((((((header_old.split("OS=")[1]).split("=")[0]).replace("/","-")).replace("(","")).replace(")","")).replace(":","_")).replace(".","")).replace(" ", "_")).replace("__","_")).replace("_-_","_")).replace("'","")).replace("#","")
      header_org = header_org[:-3]
      outfile.write(">%s%s%s\n%s\n" % (header_accession, delimiter_char, header_org, seq))

outfile.close()
