#!/usr/bin/python
#Anja, February 2017
#for python3.5

"""
#runinng blastdbcmd using following command and outformat:
blastdbcmd  -entry_batch infile.txt -outfmt "%a;;;%T;;;%X;;;%L;;;%B;;;%K" -db /extra/databases/nr -out outfile_with-taxonomy.txt
"""


#Import modules
##########################################################################################

import sys
import argparse
import os

#ARGPARSE
##########################################################################################

parser = argparse.ArgumentParser(description='Map tax ID and domain information to a parsed output file from Diamond')
parser.add_argument('-an','--input1', metavar='tsv', required = True, help='Parsed annotation, tsv file')
parser.add_argument('-b','--input2', metavar='tsv', required = True, help='Parsed Diamond Blast file (generated using Parse_diamond_blast_results_id.py)')
parser.add_argument('-t1','--input3', metavar='tsv', required = True, help= 'accessions of blast results including all taxa with corresponding taxID and taxonomy separated by ;;; and generated using blastdbcmd, domain information in 6th column')
parser.add_argument('-t2','--input4', metavar='tsv', required = True, help= 'accessions of blast resulls without selected taxa with corresponding taxID and taxonomy separated by ;;; and generated using blastdbcmd, domain information in 6th column')
parser.add_argument('-o','--output', metavar='tsv', required = True, help= 'diamond blast ouput including taxonomy for domain')
args = parser.parse_args()


if args.input1 is None and not os.path.exists(args.input):
  print("gene product file missing!")
  sys.exit(-1)

if args.input2 is None and not os.path.exists(args.input):
    print("blast results file missing!")
    sys.exit(-1)

if args.input3 is None and not os.path.exists(args.input):
	print("taxonomy file 1 missing!")
	sys.exit(-1)

if args.input4 is None and not os.path.exists(args.input):
	print("taxonomy file 2 missing!")
	sys.exit(-1)


annotation = open(args.input1).readlines()
diamond = open(args.input2).readlines()
taxonomy1 = open(args.input3).readlines()
taxonomy2 = open(args.input4).readlines()
outfile= open("%s" % args.output,'w')



#Read data into dictionaries
##########################################################################################

def Read_accession_dict(annotation):
	Protein_dic = {}
	accessions_list = []

	for line in annotation:
		entry = line.split('\t', 1)
		accessions_list.append(entry[0].strip())
		Protein_dic[entry[0]] = entry[1].strip()

	return accessions_list, Protein_dic


def Read_blast_results(diamond):
	Blast_dic1 = {}
	Blast_dic2 = {}

	for line in diamond:
		entry = line.split('\t')
		entry1 = '\t'.join(entry[1:6])
		entry2 = '\t'.join(entry[6:11])

		Blast_dic1[entry[0]] = entry1.strip()
		Blast_dic2[entry[0]] = entry2.strip()

	return Blast_dic1, Blast_dic2


def Read_taxa_info1(taxonomy):
	Taxon_dic1 = {}

	for line in taxonomy:
		entry = line.split(';;;', )

		Taxon_dic1[entry[0]] = entry[5].strip()

	return Taxon_dic1


def Read_taxa_info2(taxonomy):
	Taxon_dic2 = {}

	for line in taxonomy:
		entry = line.split(';;;', )

		Taxon_dic2[entry[0]] = entry[5].strip()

	return Taxon_dic2



#Parse final table
##########################################################################################
"""
accesssion_list = Read_accession_dict(annotation)[0]
annotation_dic = Read_accession_dict(annotation)[1]
Taxon_dic1 = Read_taxa_info1(taxonomy)
Taxon_dic2 = Read_taxa_info1(taxonomy)
Blast_dic1, Blast_dic2 = Read_blast_results(diamond)
"""


def Parse_final_table(accession_list, annotation_dic, Blast_dic1, Blast_dic2, Taxon_dic1, Taxon_dic2):
	outfile.write("accession\torganism\tprot-length\tTopHit\ts-len\tE-value\tscore\tid\tDomain\tTopHit-wo\ts-len_wo\tE-value-wo\tscore-wo\tID-wo\tDomain_wo\n")

	for acc in accession_list:
		if acc in annotation_dic:
			product = annotation_dic[acc]
		else:
			product = "-"

		if acc in Blast_dic1:
			Blast_results1 = Blast_dic1[acc]
			hit_acc1 = Blast_results1.split()[0]

			if hit_acc1 in Taxon_dic1:
				Domain1 = Taxon_dic1[hit_acc1]
			else:
				Domain1 = '-'
		else:
			print("acc not found")

		if acc in Blast_dic2:
			Blast_results2 = Blast_dic2[acc]
			hit_acc2 = Blast_results2.split()[0]

			if hit_acc2 in Taxon_dic2:
				Domain2 = Taxon_dic2[hit_acc2]
			else:
				Domain2 = '-'
		else:
			print("acc not found")


		outfile.write("%s\t%s\t%s\t%s\t%s\t%s\n" %(acc, product, Blast_results1, Domain1, Blast_results2, Domain2))



def main():
	Parse_final_table(Read_accession_dict(annotation)[0], Read_accession_dict(annotation)[1], Read_blast_results(diamond)[0], Read_blast_results(diamond)[1], Read_taxa_info1(taxonomy1), Read_taxa_info2(taxonomy2))


if __name__ == "__main__":
	main()

outfile.close()
